import numpy as np
from PIL import Image as im
from PIL import ImageOps, ImageChops

def create_circular_mask(h, w, outer_radius, inner_radius, center=None):
    """
    Create a circular ring mask for specified radii.

    Authors:
        Weiqi Yue, Gabriel Ponon, Zhuldyz Ualikhankyzy, Nathaniel K. Tomczak

    :param h: Height of the mask.
    :type h: int
    :param w: Width of the mask.
    :type w: int
    :param outer_radius: Outer radius for the mask.
    :type outer_radius: float
    :param inner_radius: Inner radius for the mask.
    :type inner_radius: float
    :param center: Coordinate tuple of the center of the mask. Uses the middle of the image if None.
    :type center: tuple, optional
    :return: The circular mask.
    :rtype: numpy.ndarray
    """
    if center is None:
        center = (int(w/2), int(h/2))
    Y, X = np.ogrid[:h, :w]
    dist_from_center = np.sqrt((X - center[0])**2 + (Y - center[1])**2)
    mask = (dist_from_center <= outer_radius) & (dist_from_center >= inner_radius)
    return mask

def define_mask(h=2048, w=2048, inner_radius=40, white_width=8, black_width=10):
    """
    Returns a multi-ring mask that can be used to filter an XRD image.

    Authors:
        Weiqi Yue, Gabriel Ponon, Zhuldyz Ualikhankyzy, Nathaniel K. Tomczak

    :param h: Height of the mask.
    :type h: int
    :param w: Width of the mask.
    :type w: int
    :param inner_radius: Inner radius of the first ring.
    :type inner_radius: int, optional
    :param white_width: Width of the transparent ring portions.
    :type white_width: int, optional
    :param black_width: Width of the opaque ring portions.
    :type black_width: int, optional
    :return: The multi-ring mask.
    :rtype: numpy.ndarray
    """
    center = (int(w/2), int(h/2))
    result_mask = np.zeros((h, w))
    for i in range(0, 50):
        start = inner_radius + (white_width + black_width) * i
        end = start + white_width
        raw_mask = create_circular_mask(w, h, outer_radius=end, inner_radius=start)
        one_array = np.ones((w, h))
        mask = raw_mask * one_array
        result_mask = np.add(result_mask, mask)
    return result_mask

def add_margin(pil_img, top, right, bottom, left, color):
    """
    Add margins around the border of an image.

    Authors:
        Weiqi Yue, Gabriel Ponon, Zhuldyz Ualikhankyzy, Nathaniel K. Tomczak

    :param pil_img: Input PIL image to which margins are added.
    :type pil_img: PIL.Image
    :param top: Size of the top margin.
    :type top: int
    :param right: Size of the right margin.
    :type right: int
    :param bottom: Size of the bottom margin.
    :type bottom: int
    :param left: Size of the left margin.
    :type left: int
    :param color: Color to overlay for margins.
    :type color: int or tuple
    :return: The image with added margins.
    :rtype: PIL.Image
    """
    width, height = pil_img.size
    new_width = width + right + left
    new_height = height + top + bottom
    result = im.new(pil_img.mode, (new_width, new_height), color)
    result.paste(pil_img, (left, top))
    return result

def find_center(img, mask=None):
    """
    Apply a mask to the central region of interest on an XRD image and find the coordinates of maximum intensity.

    Authors:
        Weiqi Yue, Gabriel Ponon, Zhuldyz Ualikhankyzy, Nathaniel K. Tomczak

    :param img: Input PIL XRD image to find the ring center.
    :type img: PIL.Image
    :param mask: The mask to use. If None, a mask will be generated through the define_mask function.
    :type mask: numpy.ndarray, optional
    :return: The coordinates of the maximum intensity.
    :rtype: tuple
    """
    if mask is None:
        mask = define_mask()

    mask = im.fromarray(mask)

    img_padding = add_margin(img, 300, 300, 300, 300, 0)
    mask_padding = add_margin(mask, 300, 300, 300, 300, 0)

    max_i = -99999
    cord = np.nan
    p = 0

    for i in range(-20, 20):
        for j in range(-20, 20):
            offset_img = np.roll(mask_padding, i, axis=0)
            offset_img = np.roll(offset_img, j, axis=1)
            aa = img_padding * offset_img
            c = aa.sum()
            if c > max_i:
                max_i = c
                cord = [i, j]
            elif c == max_i:
                p += 1

    final_x = cord[1] + 1024
    final_y = cord[0] + 1024

    return final_x, final_y

    '''
    CHANGELOG:

    v0_1 (Jul 23 2022) 
    '''



from PIL import Image

def find_center_of_gravity(image):
    """
    Centering method by weighing every pixel using its intensity.
    Returns the coordinates of the "center of gravity" of the image.

    Authors:
        Ethan Fang
    
    :param image: Input PIL image file for centering.
    :type image: PIL.Image.Image
    :return: The weighted X and Y coordinates of the center of gravity.
    :rtype: Tuple[float, float]
    """

    grayscale_image = image.convert("L")
    width, height = grayscale_image.size
    sumX, sumY, total = 0, 0, 0

    for x in range(width):
        for y in range(height):
            intensity = grayscale_image.getpixel((x, y))
            sumX += intensity * x
            sumY += intensity * y
            total += intensity

    weightedX = sumX / total
    weightedY = sumY / total
    return weightedX, weightedY

    '''
    CHANGELOG:

    v0_1 (July 10 2023) 
    '''




from PIL import Image as im
import numpy as np
from PIL import ImageOps, ImageChops


def add_margin(pil_img, top, right, bottom, left, color):
    '''
    Image padding script, to add 300 pixels around the border of an XRD image
    so the images become 2048 plus 600 square matrices.

    Authors:
        Weiqi Yue, Gabriel Ponon, Zhuldyz Ualikhankyzy, Nathaniel K. Tomczak

    :param pil_img: Input PIL image to which margins are added.
    :type pil_img: PIL.Image.Image
    :param top: Size of the top margin.
    :type top: int
    :param right: Size of the right margin.
    :type right: int
    :param bottom: Size of the bottom margin.
    :type bottom: int
    :param left: Size of the left margin.
    :type left: int
    :param color: Color to overlay for margins.
    :type color: int or tuple
    :return: Resulting image with added margins.
    :rtype: PIL.Image.Image
    '''
    width, height = pil_img.size
    new_width = width + right + left
    new_height = height + top + bottom
    result = im.new(pil_img.mode, (new_width, new_height), color)
    result.paste(pil_img, (left, top))
    return result


def reshape(input_img):
    '''
    Change image array shape to fit resize operation.

    Authors:
        Weiqi Yue, Gabriel Ponon, Zhuldyz Ualikhankyzy, Nathaniel K. Tomczak

    :param input_img: The input PIL image to reshape.
    :type input_img: PIL.Image.Image
    :return: Reshaped image array.
    :rtype: numpy.ndarray
    '''
    result_img = input_img.resize((4194304,1))
    result_img = np.array(result_img)
    result_img = result_img[0]
    return result_img


def resize_image(img, ref_path, r_only=False, img_only=False, fixed_ratio=None):
    '''
    Resize image to desired 2048 x 2048 size by finding optimum ratio.
    The optimum is determined by minimizing a correlation coefficient between
    the input image and a reference image.

    Authors:
        Weiqi Yue, Gabriel Ponon, Zhuldyz Ualikhankyzy, Nathaniel K. Tomczak

    :param img: Input PIL image to resize.
    :type img: PIL.Image.Image
    :param ref_path: Full path to the reference image.
    :type ref_path: str
    :param r_only: True when the output is the ratio number only. (default: False)
    :type r_only: bool
    :param img_only: True when the output is the PIL image only. (default: False)
    :type img_only: bool
    :param fixed_ratio: The value to fix the ratio for computation. Set to None to retrieve the ratio value automatically. (default: None)
    :type fixed_ratio: float or None
    :return: Resized image or ratio value, depending on the parameters.
    :rtype: PIL.Image.Image or float
    '''
    # add padding to PIL image input
    im_padding = add_margin(img, 300, 300, 300, 300, 0)
    ref = im.open(ref_path)
    C2 = reshape(ref)

    old_size = im_padding.size
    desired_size = 2048

    # ratio of diameter
    if fixed_ratio is not None:
        ratio = np.asarray([fixed_ratio])  # fixed ratio
    else:
        ratio = np.arange(0.9, 1.1, 0.005)  # create a series of ratios, change parameters here

    max_cor = -99999
    max_img = np.nan

    # iterate over ratios to find configuration with the lowest correlation
    for i in ratio:
        new_size = tuple([int(x * i) for x in old_size])  # set reference size

        # resize padding image
        im_re = im_padding.resize(new_size)

        # create a new image and paste the resized one onto it
        new_im = im.new(im_padding.mode, (desired_size, desired_size))
        new_im.paste(im_re, ((desired_size - new_size[0]) // 2, (desired_size - new_size[1]) // 2))
        max_img = new_im  # set max to current since there is only one

        # skip correlation if fixed ratio is not defined
        if fixed_ratio is None:
            # calculate correlation
            C1 = reshape(new_im)
            cor = np.corrcoef(C1, C2)  # find correlation value between image and reference
            aa = cor[0, 1]
            if max_cor < aa:
                max_cor = aa
                max_img = new_im
                r = i  # set min correlation ratio value as output ratio
        else:
            r = i

    if r_only:
        return r
    elif img_only:
        return max_img
    else:
        return max_img, r


'''
CHANGELOG:

v0_1 (Jul 23 2022) - changed ratio range from 0.95 to 1.05 to 0.9 to 1.1
                   - fixed bad condition for fixed ratio
'''



import numpy as np
from PIL import Image as im


def dark_correction(img, dark_file_path, threshold=16384):
    '''
    Apply darkness correction to the specified input file and output
    a dark-corrected PIL image.

    Authors: 
        Weiqi Yue, Gabriel Ponon, Zhuldyz Ualikhankyzy, Nathaniel K. Tomczak

    :param img: Input file image for dark correction.
    :type img: PIL.Image.Image
    :param dark_file_path: Full path to reference image whose pixels are subtracted from those of the input images.
    :type dark_file_path: str
    :param threshold: The intensity value below which the pixels are set to 0. (default: 16384)
    :type threshold: int
    :return: Dark-corrected PIL image.
    :rtype: PIL.Image.Image
    '''
    img_arr = np.array(img)  # open image as array
    img_dark = np.array(im.open(dark_file_path))  # open reference image as array

    # dark correction: subtract original image
    darkCor = img_arr - img_dark

    # set all negative values as 0
    darkCor[darkCor > threshold] = 0  # set threshold at given number
    save_img = im.fromarray(darkCor)
    return save_img

    '''
    CHANGELOG:

    v0_1 (Jul 23 2022) 
    '''



from PIL import Image as im
import numpy as np

def register_image(img, input_center_x, input_center_y):
  '''
    Outputs a PIL image shifted so that the specified 'true' center
    coordinates can be found in the geometric center of the image frame.
    The image is cropped to remove trailing regions and resized to the 
    original image size.

    Authors:
        Weiqi Yue, Gabriel Ponon, Zhuldyz Ualikhankyzy, Nathaniel K. Tomczak

    :param img: The input PIL image to center and register.
    :type img: PIL.Image
    :param input_center_x: The 'true' center x-coordinate identified for the image.
    :type input_center_x: float
    :param input_center_y: The 'true' center y-coordinate identified for the image.
    :type input_center_y: float
'''

  
  ## Define the center of the image and the offset between it and the center of the ring
  dim_x = img.size[0]
  dim_y = img.size[1]
  true_center_x = dim_x/2
  true_center_y = dim_y/2
  offset_x = int(round(true_center_x - input_center_x))
  offset_y = int(round(true_center_y - input_center_y))

  ### move rings to the center of the image and use dark pixel 
  offset_img = np.roll(img, offset_y, axis=0)
  offset_img = np.roll(offset_img, offset_x, axis=1)
  offset_image = im.fromarray(offset_img)

  # capture non-cropped section of the image and save to new variable
  if offset_y >= 0 and offset_x <= 0:
  # assign new array to contents of desired image
    new_img = offset_img[offset_y:dim_y, 0: dim_x + offset_x]
  elif offset_y < 0 and offset_x <= 0:
    new_img = offset_img[0:dim_y + offset_y, 0: dim_x + offset_x]
  elif offset_y >= 0 and offset_x > 0:
    new_img = offset_img[offset_y: dim_y, offset_x:dim_x]
  else:
    new_img = offset_img[0:dim_y + offset_y, offset_x: dim_x]

  # convert pixel value array into image
  new_img = im.fromarray(new_img)

  # resize the image
  new_img = new_img.resize((dim_x, dim_y))
  return new_img

  '''
  CHANGELOG:

  v0_1 (Jul 23 2022) 
  '''


from PIL import Image
import os
from os.path import join, isfile, isdir
import re
import sys

# relative imports
from dark_correction import dark_correction
from find_center import find_center
from image_registration import register_image
from temperature_independent import resize_image

def save_img(img, file_path):
    ''' 
    Save image as tiff file to specified file path.

    Authors:
        Weiqi Yue, Gabriel Ponon, Zhuldyz Ualikhankyzy, Nathaniel K. Tomczak

    :param img: PIL image to save.
    :type img: PIL.Image
    :param file_path: Complete file path for where to save the file.
    :type file_path: str
    :param is_dry_run: Specify whether to perform a dry-run operation (default: False).
    :type is_dry_run: bool
    '''
    # save the file on different directory and filename with categorical suffix
    final_file_path = file_path.replace('darkCor_', '').replace('.tiff', '_v00_0.tiff')  # TODO: change to regex
    print('saving result to path --> {}'.format(final_file_path))
    img.save(final_file_path)

    '''
    CHANGELOG:

    v0_1 (Jul 23 2022) 
    '''

def image_operations(file_path, output_folder, dark_file_path, ref_img_path):
    '''
    Apply preprocessing transformations to image and save to disk.

    Authors:
        Weiqi Yue, Gabriel Ponon, Zhuldyz Ualikhankyzy, Nathaniel K. Tomczak

    :param file_path: The full file path to the image input.
    :type file_path: str
    :param output_folder: The folder for which to save the outputs.
    :type output_folder: str
    :param dark_file_path: The full path to the dark file to be used for correction.
    :type dark_file_path: str
    :param ref_img_path: Full path to reference image to be used for resizing.
    :type ref_img_path: str
    '''
    try:
        curr_img = Image.open(file_path)  # open image as numpy array
        curr_img = dark_correction(curr_img, dark_file_path)  # TODO: add dark file path as required
        img_center_x, img_center_y = find_center(curr_img)  # find the ring center
        curr_img = register_image(curr_img, img_center_x, img_center_y)  # register image given center
        curr_img, ratio_value = resize_image(curr_img, ref_path=ref_img_path)  # resize the image and get the rescale ratio

        print(f'file path: {file_path}')
        print(f'image center: ({img_center_x}, {img_center_y})')
        print(f'ratio: {ratio_value}')

        file_name = re.findall('\w+.tiff$', file_path)[0]  # get the file name of the image
        save_path = join(output_folder, file_name)
        save_img(curr_img, save_path)  # save the image
    except Exception as e:
        print(e)

    '''
    CHANGELOG:

    v0_1 (Jul 23 2022) 
    '''



import cv2
import pandas as pd
import numpy as np

def fit_ellipse(xcoord, ycoord):
    """
    Fit an ellipse to the given cluster points and extract ellipse parameters.

    Authors: 
        Ethan Fang

    :param xcoord: X-coordinates of the cluster points.
    :type xcoord: list or numpy.ndarray
    :param ycoord: Y-coordinates of the cluster points.
    :type ycoord: list or numpy.ndarray
    :return: DataFrame containing ellipse information sorted by increasing major axis length.
    :rtype: pandas.DataFrame
    """
    # Create a list to store ellipse information
    ellipse_data = []

    # Iterate over each cluster
    for cluster_idx in range(len(xcoord)):
        # Get the coordinates for the current cluster
        coordinates = np.column_stack((xcoord[cluster_idx], ycoord[cluster_idx]))
        center = (1023, 1023)

        # Fit an ellipse to the coordinates
        ellipse = cv2.fitEllipse(coordinates)

        # Extract the ellipse parameters
        center, axes, angle = ellipse
        major_axis_length = max(axes)
        minor_axis_length = min(axes)

        # Calculate the eccentricity of the ellipse
        a = major_axis_length / 2.0
        b = minor_axis_length / 2.0
        eccentricity = np.sqrt(1 - (b**2 / a**2))

        # Store the ellipse information in a dictionary
        ellipse_info = {
            'Cluster': cluster_idx,
            'Eccentricity': eccentricity,
            'Center': center,
            'Major Axis Length': major_axis_length,
            'Minor Axis Length': minor_axis_length,
            'Angle': angle
        }

        # Append the dictionary to the ellipse_data list
        ellipse_data.append(ellipse_info)

    # Create a DataFrame from the ellipse_data list
    df_ellipse = pd.DataFrame(ellipse_data)

    # Sort the DataFrame by increasing major axis length
    df_ellipse = df_ellipse.sort_values('Major Axis Length')

    return df_ellipse

    '''
    CHANGELOG:

    v0_1 (July 10 2023) 
    '''



import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import imageio
import math
from sklearn.cluster import KMeans
from PIL import Image

def process_clusters(image_path, percent=0.15, number_cluster=15):
    """
    Given the image path, a percent for threshold, and an estimate number of clusters,
    obtain a dataframe of all clusters.

    Authors:
        Ethan Fang

    :param image_path: The path to the input image file.
    :type image_path: str
    :param percent: The percentage of pixels to consider, defaults to 0.15.
    :type percent: float, optional
    :param number_cluster: The number of clusters to find, defaults to 15.
    :type number_cluster: int, optional
    :return: The DataFrame with the processed results.
    :rtype: pandas.DataFrame
    """

    df = find_ring_coordinates(image_path, percent)
    df = find_clusters(df, number_cluster)

    return df

    '''
    CHANGELOG:

    v0_1 (July 10 2023) 
    '''

def calculate_distance(x1, y1, x2, y2):
    """
    Calculate the Euclidean distance between two points.

    Authors:
        Ethan Fang

    :param x1: X-coordinate of the first point.
    :type x1: float
    :param y1: Y-coordinate of the first point.
    :type y1: float
    :param x2: X-coordinate of the second point.
    :type x2: float
    :param y2: Y-coordinate of the second point.
    :type y2: float
    :return: The Euclidean distance between the two points.
    :rtype: float
    """
    distance = math.sqrt((x2 - x1)**2 + (y2 - y1)**2)
    return distance

    '''
    CHANGELOG:

    v0_1 (July 10 2023) 
    '''


def find_ring_coordinates(image_path, percent=0.15):
    """
    Find the coordinates of pixels with the highest intensities in an image.

    Authors:   
        Ethan Fang

    :param image_path: The path to the input image file.
    :type image_path: str
    :param percent: The percentage of pixels to consider, defaults to 0.15.
    :type percent: float, optional
    :return: A DataFrame containing the coordinates and intensities of the selected pixels.
    :rtype: pandas.DataFrame
    """
    image = Image.open(image_path).convert("L")
    intensities = np.array(image).flatten()
    intensities_normalized = intensities / 16383.0

    num_pixels = int(len(intensities_normalized) * percent)
    top_indices = np.argpartition(intensities_normalized, -num_pixels)[-num_pixels:]

    # Extract pixel coordinates and intensities
    pixels = []
    width, height = image.size
    for index in top_indices:
        y, x = divmod(index, width)
        intensity = intensities_normalized[index]
        pixels.append((int(x), int(y), intensity))
        
    df = pd.DataFrame(pixels, columns=["x", "y", "intensity"])
    
    # Obtaining coordinates
    threshold = df["intensity"][0]

    image = np.array(Image.open(image_path))

    distances, distanceCounter, xCoordinates, yCoordinates = [], [], [], []
    pixelNumber = 0
    mask = np.zeros_like(image, dtype=bool)
    for index, row in df.iterrows():
        x = int(row['x'])
        y = int(row['y'])
        distances.append(calculate_distance(x, y, 1023, 1023))
        distanceCounter.append(pixelNumber)
        image[x, y] = 1
        mask[x, y] = True
        pixelNumber += 1

    # Set the rest of the pixels not in the mask equal to 0
    image[~mask] = 0

    # Finish the dataframe
    distances, distanceCounter, df["x"], df["y"] = zip(*sorted(zip(distances, distanceCounter, df["x"], df["y"])))
    df['Pixel Number'] = distanceCounter
    df['Distance'] = distances
    df = df[df['Distance'] > 400]
    df = df.reset_index(drop=True)
    return df
    
    '''
    CHANGELOG:

    v0_1 (July 10 2023) 
    '''


def find_clusters(df, number_cluster=15):
    """
    Find clusters in the given DataFrame using K-means clustering.

    Authors:
        Ethan Fang

    :param df: The DataFrame containing the coordinates and intensities of the pixels.
    :type df: pandas.DataFrame
    :param number_cluster: The number of clusters to find, defaults to 15.
    :type number_cluster: int, optional
    :return: The DataFrame with an additional 'Cluster' column indicating the cluster label for each pixel.
    :rtype: pandas.DataFrame
    """
    distanced = df['Distance'].values.reshape(-1, 1)

    kmeans = KMeans(number_cluster)
    kmeans.fit(distanced)

    cluster_labels = kmeans.labels_
    df['Cluster'] = cluster_labels

    # Sort the clusters by their size
    cluster_sizes = []
    for label in np.unique(cluster_labels):
        cluster_size = np.sum(cluster_labels == label)
        cluster_sizes.append((label, cluster_size))
    cluster_sizes.sort(key=lambda x: x[1])
    cmap = plt.cm.tab20
    kmeansX, kmeansY = [], []
    # Plot each cluster on the image
    for i, (label, color) in enumerate(zip(np.unique(cluster_labels), cmap.colors)):
        cluster_points = df[df['Cluster'] == label]
        # Do something with the cluster points, e.g., kmeansX.append(cluster_points['x'].values)
    return df

    '''
    CHANGELOG:

    v0_1 (July 10 2023) 
    '''



from setuptools import setup


setup(name='XRDimage',
      version='0.0.1.5',
      description='XRDImage is a Python 3 package developed by the SDLE Research Center at Case Western Reserve University in Cleveland, OH.',
      long_description="""
                  XRDImage is a Python3 package developed by the SDLE Research Center at Case Western Reserve University in Cleveland, OH. 
                  X-Ray Diffraction (XRD) is a technique used to identify and quantify crystalline phases in a material. By obtaining diffraction patterns, 
                  XRD enables the identification of crystalline phases and orientation, determination of various structural properties such as lattice parameters, 
                  stain, grain size, epitaxy, phase composition, and preferred orientation. The analysis of diffraction patterns also allows for 
                  the identification of internal stress and defects in crystals, providing valuable insights into material performance in different environments.

                  Due to the nature of XRD Imaging techniques, most XRD images may not turn out perfect out of the lab. In XRDImage, 
                  we have created an image processing pipeline with multitudes of functions to pre-process XRD Images.
                   """,
      url='http://engineering.case.edu/centers/sdle/',
      author='Weiqi Yue, Ethan Fang, Gabriel Ponon, Zhuldyz Ualikhankyzy, Nathaniel K. Tomczak, Pawan K. Tripathi, Roger H. French',
      author_email='wxy215@case.edu, pkt19@case.edu, roger.french@case.edu',
      license='MIT License',
      packages=[''],
      package_dir={'XRDimage': './XRDimage'},
#      package_data={'XRDimage': ['data','files/data/FullSizeModules/*','files/tutorials/*','files/data/out','README.rst']},
      python_requires='>=3.8',
      install_requires=['numpy', 'Pillow', 'opencv-python', 'matplotlib', 'imageio', 'scikit-learn'],
#      include_package_data=True,
      project_urls={"Documentation":"https://xrdimage-doc.readthedocs.io/en/latest/","Bugtracker": "https://bitbucket.org/cwrusdle/xrd-image/src/main/"},
      )




import numpy as np
from PIL import Image as im
from PIL import ImageOps, ImageChops

from .find_center import create_circular_mask

def define_mask(h=2048, w=2048, inner_radius=40, white_width=8, black_width=10):
    '''
    Returns a multiring mask of the given dimensions that can be multiplied to filter an XRD image.

    Authors:
        Weiqi Yue, Gabriel Ponon, Zhuldyz Ualikhankyzy, Nathaniel K. Tomczak

    :param h: Height of the image.
    :type h: int
    :param w: Width of the image.
    :type w: int
    :param inner_radius: Inner radius of the first ring.
    :type inner_radius: int
    :param white_width: Width of the transparent ring portions for the filter.
    :type white_width: int
    :param black_width: Width of the opaque ring portions for the filter.
    :type black_width: int
    :return: The resulting multiring mask.
    :rtype: numpy.ndarray
    '''
    # get image mask dimensions
    # define center mask value
    center = (int(w/2), int(h/2))
    # set ring mask parameters
    result_mask = np.zeros((h,w))
    # iterate over entire mask to set appropriate values
    for i in range(0,50):
        start = inner_radius + (white_width + black_width) * i
        end = start + white_width
        raw_mask = create_circular_mask(w,h, outer_radius = end, inner_radius = start)
        one_array = np.ones((w,h))
        mask = raw_mask * one_array
        result_mask = np.add(result_mask, mask)
    return result_mask

'''
CHANGELOG:

v0_1 (Jul 23 2022) 
'''
